import requests
import paramiko
import re


class Racadm:

    def __init__(self, ip, user, passwd):
        self.client = paramiko.SSHClient()
        self.client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.client.connect(ip, username=user, password=passwd, timeout=20)

    def exec_command(self, line):
        stdin, stdout, stderr = self.client.exec_command(line)
        return stdout.read().decode('utf8')


class DellEMC:

    def __init__(self, ip, user, passwd):
        self.racadm = Racadm(ip, user, passwd)
        self.ip = ip
        self.user = user
        self.passwd = passwd
        self.ret = {
            'mem': [],
            'nic': [],
            'disk': [],
            'cpu': [],
            'basic': {

            }
        }

    def _request(self, api):
        r = requests.get('https://%s%s' % (self.ip, api), verify=False, auth=(self.user, self.passwd))
        if r.status_code == 200:
            data = r.json()
            return data
        else:
            return False

    def get_ipv4(self):
        data = self.racadm.exec_command('racadm get iDRAC.IPv4')

        # print(data)
        address = re.findall(r'Address=(.*)', data)[0]
        gateway = re.findall(r'Gateway=(.*)', data)[0]
        netmask = re.findall(r'Netmask=(.*)', data)[0]
        print(address, gateway, netmask)

    def getsysinfo(self):
        data = self.racadm.exec_command("racadm getsysinfo")
        ret = {
            'os_platform': re.findall('OS Name.\s*=\s*(.+\S)\s*', data)[0],
            'os_version': re.findall('\sOS Version\s*=\s(.*)\s*', data)[0],
            'manage_ip': re.findall('Current IP Address\s*=\s*(\S*)\s*', data)[0],
        }
        return ret

    def get_redfish_basic(self):

        data = self._request('/redfish/v1/Systems/System.Embedded.1')

        ret = {
            'sn': data.get('UUID'),
            # 'sn': data.get('SerialNumber'),
            'name': data.get('HostName'),
            'model': data.get('Model'),
            'manufacturer': data.get('Manufacturer'),
            'total_memory': data['MemorySummary']['TotalSystemMemoryGiB']
        }
        return ret

    def basic(self):

        data = {}
        d1 = self.getsysinfo()
        d2 = self.get_redfish_basic()

        data.update(d1)
        data.update(d2)
        return data

    def get_disk(self):
        data = self.racadm.exec_command('racadm storage get pdisks -o')
        # print(data)

        diskdata = []
        d = []
        num = 0
        for i in data.split("\n"):

            if "Disk.Bay." in i:

                if num == 0:
                    pass
                else:
                    diskdata.append("\n".join(d))
                    d.clear()
                num += 1

            else:
                d.append("%s\n" % i)

        diskdata.append("\n".join(d))
        # print(diskdata)
        # print(num)

        return diskdata

    def disk(self):
        ret = []

        for i in self.get_disk():
            ret.append({
                'capacity': re.findall('Size.*=\s(.\S*)\s*', i)[0],
                'manufacturer': re.findall('Manufacturer.*=\s(.\S*)\s*', i)[0],
                'slot': re.findall('Name.*=\s(.+\S)\s*', i)[0],
                'sn': re.findall('SerialNumber.*=\s(.+\S)\s*', i)[0],
                'iface_type': re.findall('BusProtocol.*=\s(.+\S)\s*', i)[0]
            })

        # print(ret)
        return ret

    def memory(self):
        data = self.racadm.exec_command('racadm getsensorinfo')
        ret = []
        for i in data.split('\n'):
            if 'System Board DIMM PG' in i:
                pass
            elif "DIMM" in i:
                # print(i)
                status = re.findall(r'DIMM [A-Z]\d\s*(\w+)\s*', i)[0]
                name = re.findall('DIMM [A-Z]\d', i)[0]
                if status == 'Ok':
                    ret.append({'status': status, 'slot': name})

        return ret

    def nic(self):
        data = self.racadm.exec_command('racadm getsysinfo')

        ret = []
        for i in data.split('\n'):
            if 'NIC.' in i:
                fildata = list(filter(lambda x: x, i.split(' ')))
                name = fildata[0]
                macaddress = fildata[-1]
                ret.append({
                    'name': name,
                    'macaddress': macaddress
                })

        # print(ret)
        return ret

    def cpu(self):
        ret = []

        cpulist = self._request('/redfish/v1/Systems/System.Embedded.1/Processors')
        for i in cpulist['Members']:
            data = self._request(i['@odata.id'])
            d = {
                'model': data['Model'],
                'manufacturer': data['Manufacturer'],
                'slot': data['Socket'],
                'cores': data['TotalCores'],

            }
            ret.append(d)

        # print(ret)
        return ret

    def run(self):

        ret = self.ret
        ret['basic'] = self.basic()
        ret['mem'] = self.memory()
        ret['nic'] = self.nic()
        ret['cpu'] = self.cpu()
        ret['disk'] = self.disk()

        d3 = {
            'total_cores': sum([int(i['cores']) for i in ret['cpu']]),
            'total_disk': sum([int(float(i['capacity'])) for i in ret['disk']]),
        }
        ret['basic'].update(d3)

        # print(ret)
        return ret



if __name__ == '__main__':
    dell_obj = DellEMC(ip='172.16.10.15', user='cmdb', passwd='1qaz@WSX3edc$RFV')
    print(dell_obj.run())
    dell_obj.racadm.client.close()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

packages = ['pyesxi']

setup_args = dict(
    name="pyesxi",
    version="0.0.2",
    author="Maliao",
    author_email="maliaotw@gmail.com",
    description="DELL ESXI SDK package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Maliaotw/pyesxi.git",
    package_dir={'pyesxi': 'pyesxi'},
    packages=packages,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)

install_requires = [
    'ansible==2.8.8',
    'pysnmp>=4.4.1',
    'requests>=2.12.3',
    'PyYAML>=3.12',
    'future>=0.16.0',
    'pysnmp_mibs>=0'
]

dependency_links=[
        'https://github.com/Maliaotw/pyesxi/omsdk-1.2.445-py2.py3-none-any.whl',
]

if __name__ == '__main__':
    setuptools.setup(**setup_args, install_requires=install_requires)



# -*- coding: utf-8 -*-

#
# Dell EMC OpenManage Ansible Modules
# Version 2.0
# Copyright (C) 2018-2019 Dell Inc. or its subsidiaries. All Rights Reserved.

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

try:
    from omsdk.sdkinfra import sdkinfra
    from omsdk.sdkcreds import UserCredentials
    from omsdk.sdkfile import FileOnShare, file_share_manager
    from omsdk.sdkprotopref import ProtoPreference, ProtocolEnum
    from omsdk.http.sdkwsmanbase import WsManOptions

    HAS_OMSDK = True
except ImportError:
    HAS_OMSDK = False

from ansible.module_utils.remote_management.dellemc.dellemc_idrac import iDRACConnection


def run_get_system_inventory(idrac):
    msg = {}
    msg['changed'] = False
    msg['failed'] = False
    err = False

    try:
        # idrac.use_redfish = True
        idrac.get_entityjson()
        msg['msg'] = idrac.get_json_device()
    except Exception as e:
        err = True
        msg['msg'] = "Error: %s" % str(e)
        msg['failed'] = True
    return msg, err


def dellemc_get_system_inventory(idrac_ip,idrac_user,idrac_passwd,idrac_port):

    try:
        ANSIBALLZ_PARAMS = {
            "idrac_ip": idrac_ip,
            "idrac_user": idrac_user,
            "idrac_password": idrac_passwd,
            "idrac_port": idrac_port,
        }

        with iDRACConnection(ANSIBALLZ_PARAMS) as idrac:
            data, err = run_get_system_inventory(idrac)

        return data['msg']
    except:
        raise Exception('ERROR')


if __name__ == '__main__':
    dellemc_get_system_inventory(
        idrac_ip='171.16.1.1',
        idrac_port='2222',
        idrac_user='cmdb',
        idrac_passwd='123456'
    )()
